package MakeWorld::Interpolate;

use strict;
use warnings;

use POSIX qw[];
use MakeWorld::Global;

use String::Interpolate::Shell qw[ strinterp ];

use base qw[ Exporter ];

our @EXPORT = qw[
		    interpolate
		    interpolate_vars
	       ];

our @EXPORT_OK = qw[
		       InterpolateVars
		  ];




# various variables that may be interpolated
our %InterpolateVars;

INIT {
    @InterpolateVars{qw(uname_sysname
			uname_nodename
			uname_release
			uname_version
			uname_machine)}
      = POSIX::uname();
}


sub interpolate
{
  my ( $text, @vars ) = @_;


  return _interpolate( $text, interpolate_vars( @vars ) );
}

sub interpolate_vars {

    my ( @vars ) = @_;

    my %var = ( 
	       # Interpolate command line option values
	       builddir => $Opt->builddir,
	       cache    => $Opt->cache,
	       exec_prefix => $Opt->exec_prefix,
	       prefix      => $Opt->prefix,
	       make_cmd    => $Opt->make_cmd,

	       ( map { %{$_} } @vars ),
	       %InterpolateVars
	      );

    return \%var;
}


sub _interpolate {

    my ( $text, $var ) = @_;

    if ( 'HASH' eq ref $text )
    {
	while( my ( $key, $val ) = each %$text )
	{
	    $text->{$key} = _interpolate( $val, $var );
	}
    }

    elsif( 'ARRAY' eq ref $text )
    {
	$text = [ map { _interpolate( $_, $var ) } @{$text} ];
    }

    else
    {
	$text = strinterp( $text, $var, { undef_value => 'ignore' } );
    }


    return $text;
}



1;
